import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * Create an object with parameters calculated for single protein.
 * @author Pawel P. Wozniak
 */
public class CreateListWithParams
{
	private List<Double> params = new ArrayList<Double>();
	private static String[] titles = {"DCAspread", "SepReduction", "VarCoeff", "QuartileVarCoeff", "RobVarCoeff", "SeqLen", "MSAseq", "accVarCoeff", "helixUnique", "helixAll",
		"betaUnique", "betaAll", "otherUnique", "otherAll", "accBuriedUnique", "accBuriedAll", "accExposedUnique", "accExposedAll", "accBurBur", "accExpExp", "accExpBur",
		"accGlobal", "quartileSpread", "standDev", "skewness", "kurtosis", "ourScoreAve", "gapArea", "gapBlockSizeAve"};
	
	/**
	 * @return list of parameters in the order of 'titles' variable
	 */
	public List<Double> getParams() {
		return params;
	}

	/**
	 * @return list with names of parameters
	 */
	public static String[] getTitles() {
		return titles;
	}

	/**
	 * @param resPath - path to DCA results file;
	 * @param fastaPath - path to FASTA file;
	 * @param msaPath - path to MSA file;
	 * @param scratchSSpath - path to SCRATCH .ss file;
	 * @param scratchACCpath - path to SCRATCH .acc20 file;
	 * @param ifEachLineMSA - true - if each line in the MSA file is a separate sequence (no FASTA headers) - false otherwise;
	 * @param separation - separation between two amino acids in sequence to treat them as contacts;
	 * @param top - number of strongest predicted contacts in DCA results file to calculate parameters for;
	 * @param pos1Col - number of column in DCA result file with sequence number of first amino acid in contact (start with 0);
	 * @param pos2Col - number of column in DCA result file with sequence number of second amino acid in contact (start with 0);
	 * @param dcaCol - number of column in DCA result file with DCA result;
	 * @param splitRegex - separator of columns in DCA file;
	 */
	CreateListWithParams(String resPath, String fastaPath, String msaPath, String scratchSSpath, String scratchACCpath, boolean ifEachLineMSA, int separation, int top,
			int pos1Col, int pos2Col, int dcaCol, String splitRegex)
	{
		calcListWithParams(resPath, fastaPath, msaPath, scratchSSpath, scratchACCpath, ifEachLineMSA, separation, top, pos1Col, pos2Col, dcaCol, splitRegex);
	}
	
	/**
	 * @param resPath - path to DCA results file;
	 * @param fastaPath - path to FASTA file;
	 * @param msaPath - path to MSA file;
	 * @param scratchSSpath - path to SCRATCH .ss file;
	 * @param scratchACCpath - path to SCRATCH .acc20 file;
	 * @param ifEachLineMSA - true - if each line in the MSA file is a separate sequence (no FASTA headers) - false otherwise;
	 * @param separation - separation between two amino acids in sequence to treat them as contacts;
	 * @param top - number of strongest predicted contacts in DCA results file to calculate parameters for;
	 * @param pos1Col - number of column in DCA result file with sequence number of first amino acid in contact (start with 0);
	 * @param pos2Col - number of column in DCA result file with sequence number of second amino acid in contact (start with 0);
	 * @param dcaCol - number of column in DCA result file with DCA result;
	 */
	CreateListWithParams(String resPath, String fastaPath, String msaPath, String scratchSSpath, String scratchACCpath, boolean ifEachLineMSA, int separation, int top,
			int pos1Col, int pos2Col, int dcaCol)
	{
		calcListWithParams(resPath, fastaPath, msaPath, scratchSSpath, scratchACCpath, ifEachLineMSA, separation, top, pos1Col, pos2Col, dcaCol, ",");
	}
	
	private void calcListWithParams(String resPath, String fastaPath, String msaPath, String scratchSSpath, String scratchACCpath, boolean ifEachLineMSA, int separation, int top,
			int pos1Col, int pos2Col, int dcaCol, String splitRegex)
	{	
		try
		{
			//Get sorted results from .gplmdca file:
			TopMethods topMeth = new TopMethods();
			topMeth.gatherSortedGplmdca(resPath, pos1Col, pos2Col, dcaCol, splitRegex);
			int[] gdcaSortInd = topMeth.getGdcaSortInd();
			double[] gdcaSortVal = topMeth.getGdcaSortVal();
			List<Integer> posDca1 = topMeth.getPosDca1();
			List<Integer> posDca2 = topMeth.getPosDca2();
			List<double[]> sortedDCA = new ArrayList<double[]>();
			DecimalFormat dfDca = new DecimalFormat("#.#####");
			for (int i = 0; i < gdcaSortInd.length; i++)
			{
				double[] tmp = new double[3];
				tmp[0] = posDca1.get(gdcaSortInd[i]);
				tmp[1] = posDca2.get(gdcaSortInd[i]);
				tmp[2] = Double.parseDouble(dfDca.format(gdcaSortVal[i]));
				sortedDCA.add(tmp);
			}
			
			FastaFileReader fastaReader = new FastaFileReader(fastaPath, 1);
			String seq = fastaReader.getFastaSeqs().get(0);			
			int seqLen = seq.length();
			
			List<Character> ssSeq = null;
			List<Integer> accSeq = null;
			try
			{
				ssSeq = ScratchReader.ssRead(scratchSSpath, null);
				accSeq = ScratchReader.accRead(scratchACCpath, null);
				if(ssSeq == null || accSeq == null || ssSeq.size() != seq.length() || accSeq.size() != seq.length())
				{
					System.out.println("Bug in SCRATCH reader!");
					return;
				}
			}
			catch(Exception exDssp)
			{
				exDssp.printStackTrace();
				return;
			}
			
			ParamAnalysis paramAnalObject = new ParamAnalysis();
			paramAnalObject.mSASeqNumberCheck(msaPath, seq, ifEachLineMSA);
			paramAnalObject.runSpreadAnalysis(separation, sortedDCA, top, 0, 1, 2);
			paramAnalObject.runSeparationAnalysis(separation, sortedDCA, top, 0, 1);
			
			double helixIfCreate = 0;
			double helixHowManyCreate = 0;
			double betaIfCreate = 0;
			double betaHowManyCreate = 0;
			double otherIfCreate = 0;
			double otherHowManyCreate = 0;
			
			paramAnalObject.runPredictedAccAnalysis(separation, sortedDCA, top, 0, 1, accSeq);			
			paramAnalObject.runPredictedExposeAnalysis(separation, sortedDCA, top, 0, 1, accSeq);
			paramAnalObject.runPredictedSsAnalysis(separation, sortedDCA, top, 0, 1, "GHI", ssSeq);
			helixIfCreate = paramAnalObject.getSsIfCreate();
			helixHowManyCreate = paramAnalObject.getSsHowManyCreate();
			paramAnalObject.runPredictedSsAnalysis(separation, sortedDCA, top, 0, 1, "EB", ssSeq);
			betaIfCreate = paramAnalObject.getSsIfCreate();
			betaHowManyCreate = paramAnalObject.getSsHowManyCreate();
			paramAnalObject.runPredictedSsAnalysis(separation, sortedDCA, top, 0, 1, "CST#- ", ssSeq);
			otherIfCreate = paramAnalObject.getSsIfCreate();
			otherHowManyCreate = paramAnalObject.getSsHowManyCreate();
			
			params.add(paramAnalObject.getDcaSpread());
			params.add(paramAnalObject.getSepReduction());
			params.add(paramAnalObject.getVarCoeff());
			params.add(paramAnalObject.getQuartileVarCoeff());
			params.add(paramAnalObject.getRobVarCoeff());
			params.add((double) seqLen);
			params.add((double) paramAnalObject.getMSAseqNmb());
			params.add(paramAnalObject.getAccVarCoeff());
			params.add(helixIfCreate);
			params.add(helixHowManyCreate);
			params.add(betaIfCreate);
			params.add(betaHowManyCreate);
			params.add(otherIfCreate);
			params.add(otherHowManyCreate);
			params.add(paramAnalObject.getAccIfCreateBuried());
			params.add(paramAnalObject.getAccHowManyCreateBuried());
			params.add(paramAnalObject.getAccIfCreateExposed());
			params.add(paramAnalObject.getAccHowManyCreateExposed());
			params.add(paramAnalObject.getAccBurBur());
			params.add(paramAnalObject.getAccExpExp());
			params.add(paramAnalObject.getAccExpBur());
			params.add(paramAnalObject.getAccGlobal());
			params.add(paramAnalObject.getQuartileSpread());
			params.add(paramAnalObject.getStandDev());
			params.add(paramAnalObject.getSkewness());
			params.add(paramAnalObject.getKurtosis());
			params.add(paramAnalObject.getOurScoreAve());
			params.add(paramAnalObject.getGapArea());
			params.add(paramAnalObject.getGapBlockSizeAve());
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
}
