import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;

/**
 * Calculates each parameter used in: PPWozniak, BM Konopka, GVriend, MKotulska, "Forecasting residue-residue contact prediction accuracy"
 * @author Pawel P. Wozniak
 */
public class ParamAnalysis
{
	private String aaSymbols = "ARDNCEQGHILKMFPSTWYV";
	//private int[] aaSurfArea = {115, 225, 150, 160, 135, 190, 180, 75, 195, 175, 170, 200, 185, 210, 145, 115, 140, 255, 230, 155}; //Chothia (1975)
	//private int[] aaSurfArea = {106, 248, 163, 157, 135, 194, 198, 84, 184, 169, 164, 205, 188, 197, 136, 130, 142, 227, 222, 142}; //Rost (1995)
	//private int[] aaSurfArea = {129, 274, 193, 195, 167, 223, 225, 104, 224, 197, 201, 236, 224, 240, 159, 155, 172, 285, 263, 174}; //Tien (2013)
	private int[] aaSurfArea = {180, 312, 228, 239, 156, 267, 263, 158, 253, 227, 233, 280, 253, 271, 200, 239, 216, 300, 300, 218}; //maxes of SCOP domains
	//private int[] aaSurfArea = {212, 365, 274, 279, 242, 305, 306, 190, 309, 288, 285, 320, 302, 322, 243, 234, 257, 369, 348, 260}; //Pymol singles residues
	
	private double TPrate; //TPrates
	private double quartileVarCoeff; //quartileVarCoeff ((suma(|x-IQR(x)|)/n)/IQR(x))
	private double robVarCoeff; //robVariationCoeff ((suma(|x-mediana(x)|)/n)/mediana(x))
	private double dcaSpread; //dcaSpread
	private double varCoeff; //variationCoeff
	private double quartileSpread; // quartileSpread
	private double standDev; //standDev
	private double aaReduced; //sepReduction
	private double accVarCoeff; //accVarCoeff
	private double accIfCreateBuried; //accIfCreateBuried
	private double accHowManyCreateBuried; //accHowManyCreateBuried
	private double accIfCreateExposed; //accIfCreateExposed
	private double accHowManyCreateExposed; //accHowManyCreateExposed
	private double accBurBur; //accBurBur
	private double accExpExp; //accExpExp
	private double accExpBur; //accExpBur
	private double accGlobal; //accGlobal
	private double ssIfCreate; //ssIfCreate
	private double ssHowManyCreate; //ssHowManyCreate
	private double skewness;
	private double kurtosis;
	private double pValAve;
	private double eValAve;
	private double scoreAve;
	private double ourScoreAve; //average of: Pairwise scores are simply the number of identities between the two sequences, divided by the length of the alignment, and represented as a percentage.
	private double gapArea; //area of gaps in the MSA
	private double gapBlockSizeAve; //average of average sizes of gap blocks within each seq from MSA
	private int mSASeqNmb;
	
	/**
	 * @return identScoreAve
	 */
	public double getOurScoreAve() {
		return ourScoreAve;
	}
	
	/**
	 * @return gapArea
	 */
	public double getGapArea() {
		return gapArea;
	}

	/**
	 * @return aveBlock
	 */
	public double getGapBlockSizeAve() {
		return gapBlockSizeAve;
	}

	/**
	 * @return skewness
	 */
	public double getSkewness() {
		return skewness;
	}

	/**
	 * @return kurtosis
	 */
	public double getKurtosis() {
		return kurtosis;
	}

	/**
	 * @return accBurBur
	 */
	public double getAccBurBur() {
		return accBurBur;
	}

	/**
	 * @return accExpExp
	 */
	public double getAccExpExp() {
		return accExpExp;
	}

	/**
	 * @return accExpBur
	 */
	public double getAccExpBur() {
		return accExpBur;
	}

	/**
	 * @return accGlobal
	 */
	public double getAccGlobal() {
		return accGlobal;
	}

	/**
	 * @return accExposedUnique
	 */
	public double getAccIfCreateExposed() {
		return accIfCreateExposed;
	}

	/**
	 * @return accExposedAll
	 */
	public double getAccHowManyCreateExposed() {
		return accHowManyCreateExposed;
	}

	/**
	 * @return accBuriedUnique
	 */
	public double getAccIfCreateBuried() {
		return accIfCreateBuried;
	}

	/**
	 * @return accBuriedAll
	 */
	public double getAccHowManyCreateBuried() {
		return accHowManyCreateBuried;
	}

	/**
	 * @return helixUnique or betaUnique depending on how method was run
	 */
	public double getSsIfCreate() {
		return ssIfCreate;
	}

	/**
	 * @return helixAll or betaAll depending on how method was run
	 */
	public double getSsHowManyCreate() {
		return ssHowManyCreate;
	}

	/**
	 * @return accCV
	 */
	public double getAccVarCoeff()
	{
		return accVarCoeff;
	}

	/**
	 * @return average of MSA p-values
	 */
	public double getpValAve() {
		return pValAve;
	}

	/**
	 * @return average of MSA e-values
	 */
	public double geteValAve() {
		return eValAve;
	}

	/**
	 * @return average of MSA score values
	 */
	public double getScoreAve() {
		return scoreAve;
	}

	/**
	 * @return MSAseq
	 */
	public int getMSAseqNmb() {
		return mSASeqNmb;
	}
	
	/**
	 * @return robIQR
	 */
	public double getQuartileVarCoeff() {
		return quartileVarCoeff;
	}

	/**
	 * @return robMed
	 */
	public double getRobVarCoeff() {
		return robVarCoeff;
	}
	
	/**
	 * @return PPV
	 */
	public double getTPrate() {
		return TPrate;
	}

	/**
	 * @return corSpread
	 */
	public double getDcaSpread() {
		return dcaSpread;
	}

	/**
	 * @return CV
	 */
	public double getVarCoeff() {
		return varCoeff;
	}

	/**
	 * @return IQR
	 */
	public double getQuartileSpread() {
		return quartileSpread;
	}

	/**
	 * @return standDev
	 */
	public double getStandDev() {
		return standDev;
	}

	/**
	 * @return sepReduction
	 */
	public double getSepReduction() {
		return aaReduced;
	}

	/**
	 * Check e-values, p-values, and score of sequences in the MSA hhr file (pValAve, eValAve, scoreAve).
	 * @param hhrPath - path to the MSA file
	 */
	public void mSAQualityFromHHRfileCheck(String hhrPath)
	{
		try
		{
			int eValCol = 2;
			int pValCol = 3;
			int scoreCol = 4;
			FileReader fr = new FileReader(hhrPath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			boolean ifStart = false;
			int howMany = 0;
			eValAve = 0;
			pValAve = 0;
			scoreAve = 0;
			while((line=br.readLine()) != null)
			{
				if(line.contains("No Hit"))
				{
					ifStart = true;
				}
				else if(ifStart && !line.isEmpty())
				{
					String[] splitter = line.substring(34).split("\\s+");
					howMany++;
					eValAve = eValAve + Double.parseDouble(splitter[eValCol]);
					pValAve = pValAve + Double.parseDouble(splitter[pValCol]);
					scoreAve = scoreAve + Double.parseDouble(splitter[scoreCol]);
				}
				else if(ifStart && line.isEmpty())
				{
					break;
				}
			}
			br.close();
			fr.close();
			
			eValAve = eValAve/howMany;
			pValAve = pValAve/howMany;
			scoreAve = scoreAve/howMany;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Check number of sequences in the MSA file and calculates some scores (mSASeqNmb, ourScoreZscore, gapArea, gapBlockSizeAve).
	 * @param msaPath - path to the MSA file
	 * @param query - query sequence
	 * @param aliFragIndex - indexes of alignment which are related to the query (start from 1)
	 * @param ifEachLine - true if there is no '>' sequence separator but each sequence is in another line
	 */
	public void mSASeqNumberCheck_AligFragment(String msaPath, String query, List<Integer> aliFragIndex, boolean ifEachLine)
	{
		try
		{
			FileReader fr = new FileReader(msaPath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			mSASeqNmb = 0;
			ourScoreAve = 0;
			gapArea = 0;
			gapBlockSizeAve = 0;
			String tmpSeq = "";
			boolean seqStart = false;
			while((line=br.readLine()) != null)
			{
				if(line.startsWith(">") || ifEachLine)
				{
					if(ifEachLine)
					{
						tmpSeq = tmpSeq + line.trim();
						tmpSeq = cutSeq(tmpSeq, aliFragIndex);
					}
					
					mSASeqNmb++;
					if(!tmpSeq.isEmpty())
					{
						if(tmpSeq.length() == query.length())
						{
							ourScoreAve = ourScoreAve + calcScore(tmpSeq, query);
							gapArea = gapArea + (tmpSeq.length()-tmpSeq.replace("-", "").length());
							gapBlockSizeAve = gapBlockSizeAve + findBlocks(tmpSeq);
						}
						else
						{
							mSASeqNmb = -1;
							ourScoreAve = -1;
							gapArea = -1;
							gapBlockSizeAve = -1;
							new Exception("Query with different length than sequence from MSA!");
							br.close();
							fr.close();
							return;
						}
					}
					seqStart = true;
					tmpSeq = "";
				}
				else if(seqStart)
				{
					tmpSeq = tmpSeq + line.trim();
					tmpSeq = cutSeq(tmpSeq, aliFragIndex);
				}
			}
			br.close();
			fr.close();
			ourScoreAve = ourScoreAve/mSASeqNmb;
			gapArea = (gapArea/(query.length()*mSASeqNmb))*100;
			gapBlockSizeAve = gapBlockSizeAve/mSASeqNmb;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	private String cutSeq(String tmpSeq, List<Integer> aliFragIndex)
	{
		String cut = "";
		for (Iterator<Integer> iterator = aliFragIndex.iterator(); iterator.hasNext();)
		{
			Integer integer = iterator.next();
			cut = cut + tmpSeq.charAt(integer-1);
		}
		return cut;
	}

	/**
	 * Check number of sequences in the MSA file and calculates some scores (mSASeqNmb, ourScoreZscore, gapArea, gapBlockSizeAve).
	 * @param msaPath - path to the MSA file
	 * @param query - query sequence
	 * @param ifEachLine - true if there is no '>' sequence separator but each sequence is in another line
	 */
	public void mSASeqNumberCheck(String msaPath, String query, boolean ifEachLine)
	{
		try
		{
			FileReader fr = new FileReader(msaPath);
			BufferedReader br = new BufferedReader(fr);
			String line;
			mSASeqNmb = 0;
			ourScoreAve = 0;
			gapArea = 0;
			gapBlockSizeAve = 0;
			String tmpSeq = "";
			boolean seqStart = false;
			while((line=br.readLine()) != null)
			{
				if(line.startsWith(">") || ifEachLine)
				{
					if(ifEachLine)
					{
						tmpSeq = tmpSeq + line.trim();
					}
					
					mSASeqNmb++;
					if(!tmpSeq.isEmpty())
					{
						if(tmpSeq.length() == query.length())
						{
							ourScoreAve = ourScoreAve + calcScore(tmpSeq, query);
							gapArea = gapArea + (tmpSeq.length()-tmpSeq.replace("-", "").length());
							gapBlockSizeAve = gapBlockSizeAve + findBlocks(tmpSeq);
						}
						else
						{
							mSASeqNmb = -1;
							ourScoreAve = -1;
							gapArea = -1;
							gapBlockSizeAve = -1;
							new Exception("Query with different length than sequence from MSA!");
							br.close();
							fr.close();
							return;
						}
					}
					seqStart = true;
					tmpSeq = "";
				}
				else if(seqStart)
				{
					tmpSeq = tmpSeq + line.trim();
				}
			}
			br.close();
			fr.close();
			ourScoreAve = ourScoreAve/mSASeqNmb;
			gapArea = (gapArea/(query.length()*mSASeqNmb))*100;
			gapBlockSizeAve = gapBlockSizeAve/mSASeqNmb;
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	private double findBlocks(String tmpSeq)
	{
		String[] splitter = tmpSeq.split("[a-zA-Z]+");
		double gapBlockSizeAveTMP = 0;
		int numberOfBlocks = 0;
		for(int i=0; i<splitter.length; i++)
		{
			if(!splitter[i].isEmpty())
			{
				gapBlockSizeAveTMP = gapBlockSizeAveTMP + splitter[i].length();
				numberOfBlocks++;
			}
		}
		
		if(numberOfBlocks == 0)
		{
			return 0.0;
		}
		else
		{
			return ((gapBlockSizeAveTMP/numberOfBlocks)/tmpSeq.length())*100;
		}
	}

	private double calcScore(String tmpSeq, String query)
	{
		int identities = 0;
		for (int i = 0; i < tmpSeq.length(); i++)
		{
			if(tmpSeq.charAt(i) == query.charAt(i))
			{
				identities++;
			}
		}
		return ((double) identities/tmpSeq.length())*100;
	}

	/**
	 * Analyze solvent accessibility coefficient of variation (accVarCoeff) in file.
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param acc1Col - number of first column with solvent ACC to analyze (start with 0)
	 * @param acc2Col - number of second column with solvent ACC to analyze (start with 0)
	 * @throws Exception 
	 */
	public void runAccAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int acc1Col, int acc2Col) throws Exception
	{
		try
		{
			try
			{
				accVarCoeff = 0;
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				String[] splitter = null;
				DescriptiveStatistics allACC = new DescriptiveStatistics();
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						double acc1 = Double.parseDouble(splitter[acc1Col]);
						double acc2 = Double.parseDouble(splitter[acc2Col]);
						if(acc1 >= 0 && acc2 >= 0)
						{
							allACC.addValue((acc1 + acc2)/2);
						}
						else
						{
							System.out.println("accVarCoeff: At least one ACC value is lower than 0 !!!");
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				accVarCoeff = allACC.getStandardDeviation()/allACC.getMean();
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze solvent accessibility coefficient of variation (accVarCoeff) for ACC predicted with ACCpro.
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param data - list with data to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number in data of first column of position to analyze (start with 0)
	 * @param pos2Col - number in data of second column of position to analyze (start with 0)
	 * @param accSeq - predicted values of ACC
	 * @throws Exception 
	 */
	public void runPredictedAccAnalysis(int separation, List<double[]> data, int top, int pos1Col, int pos2Col, List<Integer> accSeq) throws Exception
	{
		try
		{
			accVarCoeff = 0;
			int counter = 0;
			DescriptiveStatistics allACC = new DescriptiveStatistics();
			Iterator<double[]> iterator = data.iterator();
			while(data.iterator().hasNext() && counter < top)
			{
				double[] dataTable = (double[]) iterator.next();
				int pos1 = (int) dataTable[pos1Col];
				int pos2 = (int) dataTable[pos2Col];
				if(Math.abs(pos1-pos2) >= separation)
				{
					double acc1 = accSeq.get(pos1-1);
					double acc2 = accSeq.get(pos2-1);
					if(acc1 >= 0 && acc2 >= 0)
					{
						allACC.addValue((acc1 + acc2)/2);
					}
					else
					{
						System.out.println("accVarCoeff: At least one ACC value is lower than 0 !!!");
					}
					counter++;
				}
			}
			
			if(counter<top)
			{
				throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
			}
			
			accVarCoeff = allACC.getStandardDeviation()/allACC.getMean();
		}
		catch(NumberFormatException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze solvent accessibility coefficient of variation (accVarCoeff) for ACC predicted with ACCpro.
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param fastaPdbNmb - PDB numbers of sequence for which ACC was predicted (if null then mapping is equal)
	 * @param accSeq - predicted values of ACC
	 * @throws Exception 
	 */
	public void runPredictedAccAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, List<Integer> fastaPdbNmb, List<Integer> accSeq) throws Exception
	{
		try
		{
			try
			{
				if(fastaPdbNmb == null)
				{
					fastaPdbNmb = new ArrayList<Integer>();
					for (int i = 0; i < accSeq.size(); i++)
					{
						fastaPdbNmb.add(i+1);
					}
				}
				accVarCoeff = 0;
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				String[] splitter = null;
				DescriptiveStatistics allACC = new DescriptiveStatistics();
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						int accPos1 = getPDBnmbPos(fastaPdbNmb, pos1);
						int accPos2 = getPDBnmbPos(fastaPdbNmb, pos2);
						if(accPos1 >= 0 && accPos2 >= 0)
						{
							double acc1 = accSeq.get(accPos1);
							double acc2 = accSeq.get(accPos2);
							if(acc1 >= 0 && acc2 >= 0)
							{
								allACC.addValue((acc1 + acc2)/2);
							}
							else
							{
								System.out.println("accVarCoeff: At least one ACC value is lower than 0 !!!");
							}
						}
						else
						{
							System.out.println("accVarCoeff: Wrong PDB number in the DCA results list!");
							accVarCoeff = -1;
							br.close();
							fr.close();
							return;
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				accVarCoeff = allACC.getStandardDeviation()/allACC.getMean();
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	private int getPDBnmbPos(List<Integer> fastaPdbNmb, int pos1)
	{
		for (int i = 0; i < fastaPdbNmb.size(); i++)
		{
			if(pos1 == fastaPdbNmb.get(i))
			{
				return i;
			}
		}
		return -1;
	}

	/**
	 * Analyze relative solvent accessibility coefficient of variation (accVarCoeff).
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param acc1Col - number of first column with solvent ACC to analyze (start with 0)
	 * @param acc2Col - number of second column with solvent ACC to analyze (start with 0)
	 * @param aa1Col - number of first column with residue type to analyze (start with 0)
	 * @param aa2Col - number of second column with residue type to analyze (start with 0)
	 * @throws Exception 
	 */
	public void runRelatAccAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int acc1Col, int acc2Col, int aa1Col, int aa2Col) throws Exception
	{
		try
		{
			try
			{
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				String[] splitter = null;
				DescriptiveStatistics allACC = new DescriptiveStatistics();
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						double rsa1 = calcRSA(splitter[aa1Col].trim().charAt(0), Double.parseDouble(splitter[acc1Col]), aaSymbols, aaSurfArea);
						double rsa2 = calcRSA(splitter[aa2Col].trim().charAt(0), Double.parseDouble(splitter[acc2Col]), aaSymbols, aaSurfArea);
						if(rsa1 >= 0 && rsa2 >= 0)
						{
							allACC.addValue((rsa1 + rsa2)/2);
						}
						else
						{
							System.out.println("accVarCoeff: At least one ACC value is lower than 0 !!!");
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				accVarCoeff = allACC.getStandardDeviation()/allACC.getMean();
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze exposition of residues (accIfCreateBuried, accHowManyCreateBuried, accIfCreateExposed, accHowManyCreateExposed, accBurBur, accExpExp, accExpBur, accGlobal) on the basis of RSA predicted with ACCpro.
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param data - list with data to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number in data of first column of position to analyze (start with 0)
	 * @param pos2Col - number in data of second column of position to analyze (start with 0)
	 * @param accSeq - predicted with ACCpro relative solvent accessibility for each residue
	 * @throws Exception 
	 */
	public void runPredictedExposeAnalysis(int separation, List<double[]> data, int top, int pos1Col, int pos2Col, List<Integer> accSeq) throws Exception
	{
		double threshold = 20;
		int howManyBuried = checkHowManyBuriedPredicted(threshold, accSeq);
		int howManyExposed = checkHowManyExposedPredicted(threshold, accSeq);
		accGlobal = calcGlobalAccPredicted(accSeq);
		
		if(accGlobal < 0)
		{
			howManyBuried = -1;
			howManyExposed = -1;
		}
		
		try
		{
			List<Integer> accContact = new ArrayList<Integer>();
			int howManyUniqueBuried = 0;
			int howManyTotalBuried = 0;
			int howManyUniqueExposed = 0;
			int howManyTotalExposed = 0;
			int howManyBurBur = 0;
			int howManyExpExp = 0;
			int howManyExpBur = 0;
			
			int counter = 0;
			Iterator<double[]> iterator = data.iterator();
			while(data.iterator().hasNext() && counter < top)
			{
				double[] dataTable = (double[]) iterator.next();
				int pos1 = (int) dataTable[pos1Col];
				int pos2 = (int) dataTable[pos2Col];
				if(Math.abs(pos1-pos2) >= separation)
				{
					int ifBuried1 = checkIfBuriedPredicted(null, pos1, accSeq, threshold);
					if(ifBuried1 == 1)
					{
						if(!checkIfChecked(accContact, pos1))
						{
							howManyUniqueBuried++;
						}
						howManyTotalBuried++;
					}
					else if(ifBuried1 == 0)
					{
						if(!checkIfChecked(accContact, pos1))
						{
							howManyUniqueExposed++;
						}
						howManyTotalExposed++;
					}
					
					int ifBuried2 = checkIfBuriedPredicted(null, pos2, accSeq, threshold);
					if(ifBuried2 == 1)
					{
						if(!checkIfChecked(accContact, pos2))
						{
							howManyUniqueBuried++;
						}
						howManyTotalBuried++;
					}
					else if(ifBuried2 == 0)
					{
						if(!checkIfChecked(accContact, pos2))
						{
							howManyUniqueExposed++;
						}
						howManyTotalExposed++;
					}
					
					if(ifBuried1 == 1 && ifBuried2 == 1)
					{
						howManyBurBur++;
					}
					else if(ifBuried1 == 0 && ifBuried2 == 0)
					{
						howManyExpExp++;
					}
					else
					{
						howManyExpBur++;
					}
					
					counter++;
				}
			}
			
			if(counter<top)
			{
				throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
			}
			else
			{
				calcPredictedExposeAnalysis(howManyBuried, howManyUniqueBuried, howManyTotalBuried, howManyExposed, howManyUniqueExposed, 
						howManyTotalExposed, howManyBurBur, howManyExpExp, howManyExpBur, top);
			}
		}
		catch(NumberFormatException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze exposition of residues (accIfCreateBuried, accHowManyCreateBuried, accIfCreateExposed, accHowManyCreateExposed, accBurBur, accExpExp, accExpBur, accGlobal) on the basis of RSA predicted with ACCpro.
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param accSeq - predicted with ACCpro relative solvent accessibility for each residue
	 * @param fastaPdbNmb - PDB numbers of sequence for which ACC was predicted
	 * @throws Exception 
	 */
	public void runPredictedExposeAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, List<Integer> accSeq, List<Integer> fastaPdbNmb) throws Exception
	{
		try
		{
			double threshold = 20;
			int howManyBuried = checkHowManyBuriedPredicted(threshold, accSeq);
			int howManyExposed = checkHowManyExposedPredicted(threshold, accSeq);
			accGlobal = calcGlobalAccPredicted(accSeq);
			
			if(accGlobal < 0)
			{
				howManyBuried = -1;
				howManyExposed = -1;
			}
			
			try
			{
				List<Integer> accContact = new ArrayList<Integer>();
				int howManyUniqueBuried = 0;
				int howManyTotalBuried = 0;
				int howManyUniqueExposed = 0;
				int howManyTotalExposed = 0;
				int howManyBurBur = 0;
				int howManyExpExp = 0;
				int howManyExpBur = 0;
				
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				String[] splitter = null;
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						int ifBuried1 = checkIfBuriedPredicted(fastaPdbNmb, pos1, accSeq, threshold);
						if(ifBuried1 == 1)
						{
							if(!checkIfChecked(accContact, pos1))
							{
								howManyUniqueBuried++;
							}
							howManyTotalBuried++;
						}
						else if(ifBuried1 == 0)
						{
							if(!checkIfChecked(accContact, pos1))
							{
								howManyUniqueExposed++;
							}
							howManyTotalExposed++;
						}
						
						int ifBuried2 = checkIfBuriedPredicted(fastaPdbNmb, pos2, accSeq, threshold);
						if(ifBuried2 == 1)
						{
							if(!checkIfChecked(accContact, pos2))
							{
								howManyUniqueBuried++;
							}
							howManyTotalBuried++;
						}
						else if(ifBuried2 == 0)
						{
							if(!checkIfChecked(accContact, pos2))
							{
								howManyUniqueExposed++;
							}
							howManyTotalExposed++;
						}
						
						if(ifBuried1 == 1 && ifBuried2 == 1)
						{
							howManyBurBur++;
						}
						else if(ifBuried1 == 0 && ifBuried2 == 0)
						{
							howManyExpExp++;
						}
						else
						{
							howManyExpBur++;
						}
						
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				else
				{
					calcPredictedExposeAnalysis(howManyBuried, howManyUniqueBuried, howManyTotalBuried, howManyExposed, howManyUniqueExposed, 
						howManyTotalExposed, howManyBurBur, howManyExpExp, howManyExpBur, top);
				}		
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	public void calcPredictedExposeAnalysis(int howManyBuried, int howManyUniqueBuried, int howManyTotalBuried, int howManyExposed, 
			int howManyUniqueExposed, int howManyTotalExposed, int howManyBurBur, int howManyExpExp, int howManyExpBur, int top)
	{
		if(howManyBuried <= 0)
		{
			accIfCreateBuried = -1.0;
			accHowManyCreateBuried = -1.0;
		}
		else
		{
			accIfCreateBuried = ((double) howManyUniqueBuried)/howManyBuried;
			accHowManyCreateBuried = ((double) howManyTotalBuried)/howManyBuried;
		}
		
		if(howManyExposed <= 0)
		{
			accIfCreateExposed = -1.0;
			accHowManyCreateExposed = -1.0;
		}
		else
		{
			accIfCreateExposed = ((double) howManyUniqueExposed)/howManyExposed;
			accHowManyCreateExposed = ((double) howManyTotalExposed)/howManyExposed;
		}
		
		if(howManyBuried < 0 || howManyExposed < 0)
		{
			accBurBur = -1;
			accExpExp = -1;
			accExpBur = -1;
		}
		else
		{
			accBurBur = (double) howManyBurBur/top;
			accExpExp = (double) howManyExpExp/top;
			accExpBur = (double) howManyExpBur/top;
		}
	}
	
	/**
	 * @param fastaPdbNmb - all PDB number for sequence
	 * @param pos - which PDB number in sequence
	 * @param accSeq - ACC of sequence
	 * @param threshold - if RSA < threshold then buried
	 * @return 1 - buried, 0 - exposed, -1 - unknown
	 */
	private int checkIfBuriedPredicted(List<Integer> fastaPdbNmb, int pos, List<Integer> accSeq, double threshold)
	{
		int accPos = 0;
		if(fastaPdbNmb != null)
		{
			accPos = getPDBnmbPos(fastaPdbNmb, pos);
		}
		else
		{
			accPos = pos-1;
		}
		
		if(accPos >= 0 && accSeq.get(accPos) >= 0)
		{
			if(accSeq.get(accPos) < threshold)
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return -1;
		}
	}
	
	private int checkHowManyBuriedPredicted(double threshold, List<Integer> accSeq)
	{
		int howMany = 0;
		for (int i = 0; i < accSeq.size(); i++)
		{
			if(accSeq.get(i) < threshold && accSeq.get(i) >= 0)
			{
				howMany++;
			}
		}
		return howMany;
	}
	
	private int checkHowManyExposedPredicted(double threshold, List<Integer> accSeq)
	{
		int howMany = 0;
		for (int i = 0; i < accSeq.size(); i++)
		{
			if(accSeq.get(i) >= threshold && accSeq.get(i) >= 0)
			{
				howMany++;
			}
		}
		return howMany;
	}
	
	private double calcGlobalAccPredicted(List<Integer> accSeq) 
	{
		DescriptiveStatistics allAcc = new DescriptiveStatistics();
		int howMany = 0;
		for (int i = 0; i < accSeq.size(); i++)
		{
			if(accSeq.get(i) >= 0)
			{
				allAcc.addValue(accSeq.get(i));
			}
			else
			{
				System.out.println("accGlobal: At least one ACC value is lower than 0 !!!");
				howMany++;
			}
		}
		
		double howManyPerc = ((double) howMany)/((double) accSeq.size());
		if(howManyPerc < 0.1)
		{
			return allAcc.getMean();
		}
		else
		{
			System.out.println("accGlobal: Too many ACC unknown!!!");
			return -1;
		}
	}
	
	/**
	 * Analyze exposition of residues (accIfCreateBuried, accHowManyCreateBuried, accIfCreateExposed, accHowManyCreateExposed, accBurBur, accExpExp, accExpBur, accGlobal).
	 * @param separation - difference between to columns' values which will be taken under consideration (pos1Col and pos2Col)
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param acc1Col - number of first column with solvent ACC to analyze (start with 0)
	 * @param acc2Col - number of second column with solvent ACC to analyze (start with 0)
	 * @param aa1Col - number of first column with residue type to analyze (start with 0)
	 * @param aa2Col - number of second column with residue type to analyze (start with 0)
	 * @param accSeq - solvent accessibility for each residue
	 * @param seq - residue sequence
	 * @throws Exception 
	 */
	public void runExposeAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int acc1Col, int acc2Col, int aa1Col, int aa2Col, List<Integer> accSeq, String seq) throws Exception
	{
		try
		{
			double threshold = 20;
			int howManyBuried = checkHowManyBuried(aaSymbols, aaSurfArea, seq, threshold, accSeq);
			int howManyExposed = checkHowManyExposed(aaSymbols, aaSurfArea, seq, threshold, accSeq);
			accGlobal = calcGlobalAcc(seq, accSeq, aaSymbols, aaSurfArea);
			
			if(accGlobal < 0)
			{
				howManyBuried = -1;
				howManyExposed = -1;
			}
			
			try
			{
				List<Integer> accContact = new ArrayList<Integer>();
				int howManyUniqueBuried = 0;
				int howManyTotalBuried = 0;
				int howManyUniqueExposed = 0;
				int howManyTotalExposed = 0;
				int howManyBurBur = 0;
				int howManyExpExp = 0;
				int howManyExpBur = 0;
				
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				String[] splitter = null;
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					int acc1 = Integer.parseInt(splitter[acc1Col]);
					int acc2 = Integer.parseInt(splitter[acc2Col]);
					char aa1 = splitter[aa1Col].trim().charAt(0);
					char aa2 = splitter[aa2Col].trim().charAt(0);
					if(Math.abs(pos1-pos2) >= separation)
					{
						int ifBuried1 = checkIfBuried(aaSymbols, aa1, threshold, acc1, aaSurfArea);
						if(ifBuried1 == 1)
						{
							if(!checkIfChecked(accContact, pos1))
							{
								howManyUniqueBuried++;
							}
							howManyTotalBuried++;
						}
						else if(ifBuried1 == 0)
						{
							if(!checkIfChecked(accContact, pos1))
							{
								howManyUniqueExposed++;
							}
							howManyTotalExposed++;
						}
						
						int ifBuried2 = checkIfBuried(aaSymbols, aa2, threshold, acc2, aaSurfArea);
						if(ifBuried2 == 1)
						{
							if(!checkIfChecked(accContact, pos2))
							{
								howManyUniqueBuried++;
							}
							howManyTotalBuried++;
						}
						else if(ifBuried2 == 0)
						{
							if(!checkIfChecked(accContact, pos2))
							{
								howManyUniqueExposed++;
							}
							howManyTotalExposed++;
						}
						
						if(ifBuried1 == 1 && ifBuried2 == 1)
						{
							howManyBurBur++;
						}
						else if(ifBuried1 == 0 && ifBuried2 == 0)
						{
							howManyExpExp++;
						}
						else
						{
							howManyExpBur++;
						}
						
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				else
				{
					calcPredictedExposeAnalysis(howManyBuried, howManyUniqueBuried, howManyTotalBuried, howManyExposed, howManyUniqueExposed, 
						howManyTotalExposed, howManyBurBur, howManyExpExp, howManyExpBur, top);
				}
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	private double calcRSA(char oneAA, double oneAcc, String aaSymbols, int[] aaSurfArea)
	{
		for (int j = 0; j < aaSymbols.length(); j++)
		{
			if(oneAA == aaSymbols.charAt(j))
			{
				double rsa = (oneAcc/aaSurfArea[j])*100;
				return rsa;
			}
		}
		return -1;
	}
	
	private double calcGlobalAcc(String seq, List<Integer> accSeq, String aaSymbols, int[] aaSurfArea) 
	{
		DescriptiveStatistics allAcc = new DescriptiveStatistics();
		int howMany = 0;
		for (int i = 0; i < seq.length(); i++)
		{
			double rsa = calcRSA(seq.charAt(i), (double) accSeq.get(i), aaSymbols, aaSurfArea);
			if(rsa >= 0)
			{
				allAcc.addValue(rsa);
				if(rsa > 100)
				{
					System.out.println("RSA higher than 100%: " + i + " " + seq.charAt(i) + " " + rsa);
				}
			}
			else
			{
				System.out.println("accGlobal: At least one ACC value is lower than 0 !!!");
				howMany++;
			}
		}
		
		double howManyPerc = ((double) howMany)/((double) accSeq.size());
		if(howManyPerc < 0.1)
		{
			return allAcc.getMean();
		}
		else
		{
			System.out.println("accGlobal: Too many ACC unknown!!!");
			return -1;
		}
	}

	private int checkHowManyBuried(String aaSymbols, int[] aaSurfArea, String seq, double threshold, List<Integer> accSeq)
	{
		int howMany = 0;
		for (int i = 0; i < seq.length(); i++)
		{
			if(checkIfBuried(aaSymbols, seq.charAt(i), threshold, accSeq.get(i), aaSurfArea) == 1)
			{
				howMany++;
			}
		}
		return howMany;
	}
	
	private int checkHowManyExposed(String aaSymbols, int[] aaSurfArea, String seq, double threshold, List<Integer> accSeq)
	{
		int howMany = 0;
		for (int i = 0; i < seq.length(); i++)
		{
			if(checkIfBuried(aaSymbols, seq.charAt(i), threshold, accSeq.get(i), aaSurfArea) == 0)
			{
				howMany++;
			}
		}
		return howMany;
	}
	
	/**
	 * @param aaSymbols - all residue types
	 * @param aa - analyzed residue type
	 * @param threshold - if RSA < threshold then buried
	 * @param acc - ACC from DSSP
	 * @param aaSurfArea - max ACC for all residue types 
	 * @return 1 - buried, 0 - exposed, -1 - unknown
	 */
	private int checkIfBuried(String aaSymbols, char aa, double threshold, int acc, int[] aaSurfArea)
	{
		double rsa = calcRSA(aa, (double) acc, aaSymbols, aaSurfArea);
		if(rsa >= 0)
		{
			if(rsa < threshold)
			{
				return 1;
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return -1;
		}
	}
	
	/**
	 * Analyze TP rate in a set of files (TPrates).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column to analyze (start with 0)
	 * @param pos2Col - number of second column to analyze (start with 0)
	 * @param resCol - number of column with analyzed value (start with 0)
	 * @param resCutoff - cutoff for the analyzed value 
	 * @throws Exception 
	 */
	public void runTpAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int resCol, double resCutoff) throws Exception
	{
		try
		{
			try
			{
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				TPrate = 0.0;
				int counter = 0;
				while((line=br.readLine()) != null && counter < top)
				{
					String[] splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					double res = Double.parseDouble(splitter[resCol]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						if(res <= resCutoff)
						{
							TPrate++;
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				TPrate = (TPrate/counter)*100; //!!!!!!!!!!!!
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze TP rate in a set of files (TPrates).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column to analyze (start with 0)
	 * @param pos2Col - number of second column to analyze (start with 0)
	 * @param tpCol - number of column with indication true/false (start with 0) 
	 * @throws Exception 
	 */
	public void runTpAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int tpCol) throws Exception
	{
		try
		{
			try
			{
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				TPrate = 0.0;
				int counter = 0;
				while((line=br.readLine()) != null && counter < top)
				{
					String[] splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					int ifTp = Integer.parseInt(splitter[tpCol]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						if(ifTp == 1)
						{
							TPrate++;
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				TPrate = (TPrate/counter)*100; //!!!!!!!!!!!!
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze separation reduction in a list with data (sepReduction).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param data - list with data to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number in data of first column to analyze (start with 0)
	 * @param pos2Col - number in data of second column to analyze (start with 0)
	 * @throws Exception 
	 */
	public void runSeparationAnalysis(int separation, List<double[]> data, int top, int pos1Col, int pos2Col) throws Exception
	{
		try
		{
			aaReduced = 0.0;
			int counter = 0;
			Iterator<double[]> iterator = data.iterator();
			while(data.iterator().hasNext() && counter < top)
			{
				double[] dataTable = (double[]) iterator.next();
				int pos1 = (int) dataTable[pos1Col];
				int pos2 = (int) dataTable[pos2Col];
				if(Math.abs(pos1-pos2) >= separation)
				{
					counter++;
				}
				else
				{
					aaReduced++;
				}
			}
			
			if(counter<top)
			{
				throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
			}
		}
		catch(NumberFormatException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze separation reduction in a file (sepReduction).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column to analyze (start with 0)
	 * @param pos2Col - number of second column to analyze (start with 0)
	 * @throws Exception 
	 */
	public void runSeparationAnalysis(int separation, File file, int top, int pos1Col, int pos2Col) throws Exception
	{
		try
		{
			try
			{
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				aaReduced = 0.0;
				int counter = 0;
				while((line=br.readLine()) != null && counter < top)
				{
					String[] splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						counter++;
					}
					else
					{
						aaReduced++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze several statistics of the data column in a list with data (variationCoeff, quartileSpread, standDev, dcaSpread, robVariationCoeff, quartileVarCoeff).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param data - list with data to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number in data of first column of position to analyze (start with 0)
	 * @param pos2Col - number in data of second column of position to analyze (start with 0)
	 * @param resCol - number in data of column with analyzed result (start with 0) 
	 * @throws Exception 
	 */
	public void runSpreadAnalysis(int separation, List<double[]> data, int top, int pos1Col, int pos2Col, int resCol) throws Exception
	{
		try
		{
			int counter = 0;
			double maxDca = -1000;
			double minDca = -1000;
			DescriptiveStatistics allDCAs = new DescriptiveStatistics();
			List<Double> dcaValues = new ArrayList<Double>();
			Iterator<double[]> iterator = data.iterator();
			int tmpHowManyRows = 0;
			try
			{
				while(data.iterator().hasNext() && counter < top)
				{
					double[] dataTable = (double[]) iterator.next();
					int pos1 = (int) dataTable[pos1Col];
					int pos2 = (int) dataTable[pos2Col];
					if(Math.abs(pos1-pos2) >= separation)
					{
						double dcaVal = dataTable[resCol]; 
						if(maxDca == -1000)
						{
							maxDca = dcaVal;
						}
						
						dcaValues.add(dcaVal);
						allDCAs.addValue(dataTable[resCol]); //!!!!!!!!
						counter++;
					}
					tmpHowManyRows++;
				}
			}
			catch(NoSuchElementException nseExcep)
			{
				System.out.println("There was no next element in DCA results after " + tmpHowManyRows + " element.");
			}
			
			if(counter<top)
			{
				throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
			}
			
			minDca = dcaValues.get(dcaValues.size()-1);
			calcSpreadAnalysis(minDca, maxDca, allDCAs, dcaValues);
		}
		catch(NumberFormatException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze several statistics of the data column in a file (variationCoeff, quartileSpread, standDev, dcaSpread, robVariationCoeff, quartileVarCoeff).
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column of position to analyze (start with 0)
	 * @param pos2Col - number of second column of position to analyze (start with 0)
	 * @param resCol - number of column with analyzed result (start with 0) 
	 * @throws Exception 
	 */
	public void runSpreadAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int resCol) throws Exception
	{
		try
		{
			try
			{
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				double maxDca = -1000;
				double minDca = -1000;
				String[] splitter = null;
				DescriptiveStatistics allDCAs = new DescriptiveStatistics();
				List<Double> dcaValues = new ArrayList<Double>();
				while((line=br.readLine()) != null && counter < top)
				{
					splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						double dcaVal = Double.parseDouble(splitter[resCol]); 
						if(maxDca == -1000)
						{
							maxDca = dcaVal;
						}
						
						dcaValues.add(dcaVal);
						allDCAs.addValue(Double.parseDouble(splitter[resCol])); //!!!!!!!!
						counter++;
					}
				}
				
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				minDca = Double.parseDouble(splitter[resCol]);
				calcSpreadAnalysis(minDca, maxDca, allDCAs, dcaValues);
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	private void calcSpreadAnalysis(double minDca, double maxDca, DescriptiveStatistics allDCAs, List<Double> dcaValues)
	{
		
		dcaSpread = (maxDca-minDca)/maxDca;
		standDev = allDCAs.getStandardDeviation();
		quartileSpread = Math.abs((allDCAs.getPercentile(25)-allDCAs.getPercentile(75)))/2; //(allDCAs.getElement((counter/4)-1)-allDCAs.getElement((counter/4)*3-1))/2;
		varCoeff = standDev/allDCAs.getMean();
		double median = allDCAs.getPercentile(50);
		double robVarCoeffTMP = 0;
		double quartileSpreadTMP = 0;
		for (Iterator<Double> iterator  = dcaValues.iterator(); iterator.hasNext();)
		{
			Double oneVal = iterator.next();
			robVarCoeffTMP = robVarCoeffTMP + Math.abs(oneVal-median);
			quartileSpreadTMP = quartileSpreadTMP + Math.abs(oneVal-quartileSpread);
		}
		quartileVarCoeff = (quartileSpreadTMP/dcaValues.size())/quartileSpread;
		robVarCoeff = (robVarCoeffTMP/dcaValues.size())/median;
		skewness = allDCAs.getSkewness();
		kurtosis = allDCAs.getKurtosis();
	}
	
	/**
	 * Analyze occurrence of chosen SS in top results on the basis of SSpro prediction.
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param data - list with data to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number in data of first column to analyze (start with 0)
	 * @param pos2Col - number in data of second column to analyze (start with 0)
	 * @param ssSymbols - string with secondary structure symbols of pairs to analyze
	 * @param ssSeq - secondary structures of each residue in a sequence
	 * @throws Exception 
	 */
	public void runPredictedSsAnalysis(int separation, List<double[]> data, int top, int pos1Col, int pos2Col, String ssSymbols, List<Character> ssSeq) throws Exception
	{
		int allSS = checkHowManySSinSeq(ssSeq, ssSymbols);
		
		try
		{
			List<Integer> ssContact = new ArrayList<Integer>();
			int howManyUnique = 0;
			int howManyTotal = 0;

			int counter = 0;
			Iterator<double[]> iterator = data.iterator();
			while(data.iterator().hasNext() && counter < top)
			{
				double[] dataTable = (double[]) iterator.next();
				int pos1 = (int) dataTable[pos1Col];
				int pos2 = (int) dataTable[pos2Col];
				if(Math.abs(pos1-pos2) >= separation)
				{
					if(ssSymbols.contains(String.valueOf(ssSeq.get(pos1-1))))
					{
						if(!checkIfChecked(ssContact, pos1))
						{
							howManyUnique++;
						}
						howManyTotal++;
					}
					
					if(ssSymbols.contains(String.valueOf(ssSeq.get(pos2-1))))
					{
						if(!checkIfChecked(ssContact, pos2))
						{
							howManyUnique++;
						}
						howManyTotal++;
					}
					counter++;
				}
			}
			
			if(counter<top)
			{
				throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
			}
			
			if(allSS == 0)
			{
				ssIfCreate = -1.0;
				ssHowManyCreate = -1.0;
			}
			else
			{
				ssIfCreate = ((double) howManyUnique)/allSS;
				ssHowManyCreate = ((double) howManyTotal)/allSS;
			}
		}
		catch(NumberFormatException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze occurrence of chosen SS in top results on the basis of SSpro prediction.
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column to analyze (start with 0)
	 * @param pos2Col - number of second column to analyze (start with 0)
	 * @param ssSymbols - string with secondary structure symbols of pairs to analyze
	 * @param ssSeq - secondary structures of each residue in a sequence
	 * @param fastaPdbNmb - PDB numbers of sequence for which ACC was predicted
	 * @throws Exception 
	 */
	public void runPredictedSsAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, String ssSymbols, List<Character> ssSeq, List<Integer> fastaPdbNmb) throws Exception
	{
		try
		{
			int allSS = checkHowManySSinSeq(ssSeq, ssSymbols);
			
			try
			{
				List<Integer> ssContact = new ArrayList<Integer>();
				int howManyUnique = 0;
				int howManyTotal = 0;
				
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				while((line=br.readLine()) != null && counter < top)
				{
					String[] splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						int ss1Pos = getPDBnmbPos(fastaPdbNmb, pos1);
						if(ss1Pos >= 0)
						{
							if(ssSymbols.contains(String.valueOf(ssSeq.get(ss1Pos))))
							{
								if(!checkIfChecked(ssContact, pos1))
								{
									howManyUnique++;
								}
								howManyTotal++;
							}
						}
						else
						{
							System.out.println("Wrong PDB number in the DCA results list!");
							ssIfCreate = -1.0;
							ssHowManyCreate = -1.0;
							br.close();
							fr.close();
							return;
						}
						
						int ss2Pos = getPDBnmbPos(fastaPdbNmb, pos2);
						if(ss2Pos >= 0)
						{
							if(ssSymbols.contains(String.valueOf(ssSeq.get(ss2Pos))))
							{
								if(!checkIfChecked(ssContact, pos2))
								{
									howManyUnique++;
								}
								howManyTotal++;
							}
						}
						else
						{
							System.out.println("Wrong PDB number in the DCA results list!");
							ssIfCreate = -1.0;
							ssHowManyCreate = -1.0;
							br.close();
							fr.close();
							return;
						}
						
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				if(allSS == 0)
				{
					ssIfCreate = -1.0;
					ssHowManyCreate = -1.0;
				}
				else
				{
					ssIfCreate = ((double) howManyUnique)/allSS;
					ssHowManyCreate = ((double) howManyTotal)/allSS;
				}
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	/**
	 * Analyze occurrence of chosen SS in top results.
	 * @param separation - difference between to columns' values which will be taken under consideration
	 * @param file - file to analyze
	 * @param top - how many top results will be analyzed
	 * @param pos1Col - number of first column to analyze (start with 0)
	 * @param pos2Col - number of second column to analyze (start with 0)
	 * @param ss1Col - number of first column with secondary structure information to analyze (start with 0)
	 * @param ss2Col - number of second column with secondary structure information to analyze (start with 0)
	 * @param ssSymbols - string with secondary structure symbols of pairs to analyze
	 * @param ssSeq - secondary structures of each residue in a sequence
	 * @throws Exception 
	 */
	public void runSsAnalysis(int separation, File file, int top, int pos1Col, int pos2Col, int ss1Col, int ss2Col, String ssSymbols, List<Character> ssSeq) throws Exception
	{
		try
		{
			int allSS = checkHowManySSinSeq(ssSeq, ssSymbols);
			
			try
			{
				List<Integer> ssContact = new ArrayList<Integer>();
				int howManyUnique = 0;
				int howManyTotal = 0;
				
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line = br.readLine();
				int counter = 0;
				while((line=br.readLine()) != null && counter < top)
				{
					String[] splitter = line.split("\\s+");
					int pos1 = Integer.parseInt(splitter[pos1Col]);
					int pos2 = Integer.parseInt(splitter[pos2Col]);
					if(Math.abs(pos1-pos2) >= separation)
					{
						if(ssSymbols.contains(splitter[ss1Col].trim()))
						{
							if(!checkIfChecked(ssContact, pos1))
							{
								howManyUnique++;
							}
							howManyTotal++;
						}
						
						if(ssSymbols.contains(splitter[ss2Col].trim()))
						{
							if(!checkIfChecked(ssContact, pos2))
							{
								howManyUnique++;
							}
							howManyTotal++;
						}
						counter++;
					}
				}
				br.close();
				fr.close();
				
				if(counter<top)
				{
					throw new Exception("COUNTER EQUALS " + String.valueOf(counter));
				}
				
				if(allSS == 0)
				{
					ssIfCreate = -1.0;
					ssHowManyCreate = -1.0;
				}
				else
				{
					ssIfCreate = ((double) howManyUnique)/allSS;
					ssHowManyCreate = ((double) howManyTotal)/allSS;
				}
			}
			catch(FileNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch(NumberFormatException ex)
			{
				ex.printStackTrace();
			}
		}
		catch(IOException ex)
		{
			ex.printStackTrace();
		}
	}
	
	private int checkHowManySSinSeq(List<Character> ssSeq, String ssSymbols)
	{
		int howMany = 0;
		for (Iterator<Character> iterator = ssSeq.iterator(); iterator.hasNext();)
		{
			Character oneSS = iterator.next();
			if(ssSymbols.contains(String.valueOf(oneSS)))
			{
				howMany++;
			}
		}
		return howMany;
	}

	private boolean checkIfChecked(List<Integer> ssAll, int pos)
	{
		for (Iterator<Integer> iterator = ssAll.iterator(); iterator.hasNext();)
		{
			Integer posTmp = iterator.next();
			if(posTmp == pos)
			{
				return true;
			}	
		}
		ssAll.add(pos);
		return false;
	}
}
